__author__ = 'Lukas Leufen'
__date__ = '2020-11-05'

from mlair.data_handler.data_handler_single_station import DataHandlerSingleStation
from mlair.configuration import path_config

import logging
import os

import pandas as pd
import xarray as xr


class DataHandlerMixedSampling(DataHandlerSingleStation):

    def setup_samples(self):
        """
        Setup samples. This method prepares and creates samples X, and labels Y.
        """
        self.load_data()
        self.interpolate(dim=self.time_dim, method=self.interpolation_method, limit=self.interpolation_limit)
        self.set_inputs_and_targets()
        if self.do_transformation is True:
            self.call_transform()
        self.make_samples()

    def load_data(self):
        try:
            self.read_data_from_disk()
        except FileNotFoundError:
            self.download_data()
            self.load_data()

    def read_data_from_disk(self, source_name=""):
        """
        Load data and meta data either from local disk (preferred) or download new data by using a custom download method.

        Data is either downloaded, if no local data is available or parameter overwrite_local_data is true. In both
        cases, downloaded data is only stored locally if store_data_locally is not disabled. If this parameter is not
        set, it is assumed, that data should be saved locally.
        """
        source_name = source_name if len(source_name) == 0 else f" from {source_name}"
        path_config.check_path_and_create(self.path)
        file_name = self._set_file_name()
        meta_file = self._set_meta_file_name()
        if self.overwrite_local_data is True:
            logging.debug(f"overwrite_local_data is true, therefore reload {file_name}{source_name}")
            if os.path.exists(file_name):
                os.remove(file_name)
            if os.path.exists(meta_file):
                os.remove(meta_file)
            data, self.meta = self.download_data(file_name, meta_file)
            logging.debug(f"loaded new data{source_name}")
        else:
            try:
                logging.debug(f"try to load local data from: {file_name}")
                data = xr.open_dataarray(file_name)
                self.meta = pd.read_csv(meta_file, index_col=0)
                self.check_station_meta()
                logging.debug("loading finished")
            except FileNotFoundError as e:
                logging.debug(e)
                logging.debug(f"load new data{source_name}")
                data, self.meta = self.download_data(file_name, meta_file)
                logging.debug("loading finished")
        # create slices and check for negative concentration.
        data = self._slice_prep(data)
        self._data = self.check_for_negative_concentrations(data)
